package main

import (
	"flag"
	"fmt"
	"github.com/klipitkas/fs-go/util"
	"log"
	"net"
	"net/http"
	"os/exec"
	"strconv"
)

var port int
var dir string

func main() {

	// Handle the flags that are provided.
	flag.IntVar(&port, "port", 8080, "The port that the server will listen to.")
	flag.StringVar(&dir, "dir", ".", "The root directory that will be served.")
	// Parse the flags.
	flag.Parse()

	// Create the fileserver.
	fs := http.FileServer(http.Dir(dir))
	log.Printf("server is start %v: %v", port, dir)
	//check port
	// for i := 0; i < 5; i++ {
	// 	if checkAvailablePort(port) {
	// 		break
	// 	} else {
	// 		// log.Println(strconv.Itoa(port) + " is not available.")
	// 		log.Printf("server error on port %v: %v", port, err)
	// 		port++
	// 	}
	// }

	// Start the file server.
	err := http.ListenAndServe(":"+fmt.Sprintf("%d", port), http.HandlerFunc(handler))
	if err != nil {
		log.Fatal("server error on port %v: %v", port, err)

	}
	log.Printf("Server started on port %d\n", port)
	
	// open browser
	l, err := net.Listen("tcp", "localhost:"+strconv.Itoa(port))
	if err != nil {
		log.Fatal(err)
	}
	err = util.OpenUrl("http://localhost:" + strconv.Itoa(port))
	if err != nil {
		log.Fatal(err)
	}
	log.Fatal(http.Serve(l, fs))
}

// remove header cache
func handler(w http.ResponseWriter, r *http.Request) {
    w.Header().Set("Cache-Control", "no-cache, no-store, must-revalidate") // HTTP 1.1.
    w.Header().Set("Pragma", "no-cache") // HTTP 1.0.
    w.Header().Set("Expires", "0") // Proxies.
    
	if r.URL.Path == "/favicon.ico" {
		return
	}
	http.StripPrefix("/", http.FileServer(http.Dir(dir))).ServeHTTP(w, r)
}

//checkAvailablePort 端口可用检测,netstat 只适合linux
func checkAvailablePort(port int) (res bool) {
    checkStatement := fmt.Sprintf(`lsof -i:%d -t`, port)
    output, err := exec.Command("sh", "-c", checkStatement).Output()
    if err != nil {
        return false
    }

    if len(output) > 0 {
        return false
    }

    return true
}
